@@ -52,22 +52,16 @@ def register_removals(event_handler):
                          remove_commands=['invoke-endpoint-with-response-stream'])
     cmd_remover.remove(on_event='building-command-table.bedrock-runtime',
                          remove_commands=['invoke-model-with-response-stream',
-                                          'invoke-model-with-bidirectional-stream',
                                           'converse-stream'])
     cmd_remover.remove(on_event='building-command-table.bedrock-agent-runtime',
                          remove_commands=['invoke-agent',
                                           'invoke-flow',
                                           'invoke-inline-agent',
-                                          'optimize-prompt',
-                                          'retrieve-and-generate-stream'])
-    cmd_remover.remove(on_event='building-command-table.bedrock-agentcore',
-                        remove_commands=['invoke-code-interpreter'])
+                                          'optimize-prompt']),
     cmd_remover.remove(on_event='building-command-table.qbusiness',
                         remove_commands=['chat'])
     cmd_remover.remove(on_event='building-command-table.iotsitewise',
                         remove_commands=['invoke-assistant'])
-    cmd_remover.remove(on_event='building-command-table.logs',
-                        remove_commands=['get-log-object'])
 
 
 class CommandRemover(object):
